@@ -79,38 +79,35 @@ pub fn ha_render_forward_stage_system(universe: &mut Universe) {
                 };
                 let _ = recorder.record(RenderCommand::ActivateMesh(mesh_id));
                 let signature = info.make_material_signature(current_mesh.layout());
-                let _ = recorder.record(RenderCommand::ActivateMaterial {
-                    id: material_id,
-                    signature: signature.to_owned(),
-                });
-                let _ = recorder.record(RenderCommand::SubmitUniform {
-                    signature: signature.to_owned(),
-                    name: MODEL_MATRIX_NAME.into(),
-                    value: transform.world_matrix().into(),
-                });
-                let _ = recorder.record(RenderCommand::SubmitUniform {
-                    signature: signature.to_owned(),
-                    name: VIEW_MATRIX_NAME.into(),
-                    value: info.view_matrix.into(),
-                });
-                let _ = recorder.record(RenderCommand::SubmitUniform {
-                    signature: signature.to_owned(),
-                    name: PROJECTION_MATRIX_NAME.into(),
-                    value: info.projection_matrix.into(),
-                });
+                let _ = recorder.record(RenderCommand::ActivateMaterial(
+                    material_id,
+                    signature.to_owned(),
+                ));
+                let _ = recorder.record(RenderCommand::OverrideUniform(
+                    MODEL_MATRIX_NAME.into(),
+                    transform.world_matrix().into(),
+                ));
+                let _ = recorder.record(RenderCommand::OverrideUniform(
+                    VIEW_MATRIX_NAME.into(),
+                    info.view_matrix.into(),
+                ));
+                let _ = recorder.record(RenderCommand::OverrideUniform(
+                    PROJECTION_MATRIX_NAME.into(),
+                    info.projection_matrix.into(),
+                ));
+                for (key, value) in &material.values {
+                    let _ = recorder.record(RenderCommand::OverrideUniform(
+                        key.to_owned().into(),
+                        value.to_owned(),
+                    ));
+                }
                 if let Some(draw_options) = &material.override_draw_options {
                     let _ =
                         recorder.record(RenderCommand::ApplyDrawOptions(draw_options.to_owned()));
                 }
-                for (name, value) in &material.values {
-                    let _ = recorder.record(RenderCommand::SubmitUniform {
-                        signature: signature.to_owned(),
-                        name: name.to_owned().into(),
-                        value: value.to_owned(),
-                    });
-                }
                 // TODO: add support for virtual mesh ranges.
                 let _ = recorder.record(RenderCommand::DrawMesh(MeshDrawRange::All));
+                let _ = recorder.record(RenderCommand::ResetUniforms);
             }
 
             let _ = recorder.record(RenderCommand::SortingBarrier);
